#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::PaymentMethodGenerator
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Ipayment::API::PaymentMethodGenerator
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::PaymentMethodGenerator;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Exception;
use Test::More;
use DE_EPAGES::Ipayment::API::PaymentMethodGenerator;
use Hash::Merge qw( merge );
use DE_EPAGES::Core::API::PerlTools qw ( GenerateShortGUID );

use strict;



sub test {
    my $self = shift;
    $self->test_PaymentTypeGenerationWithUndefParams();
    $self->test_PaymentTypeGenerationWithEmptyPaymentMethodList();
    $self->test_PaymentTypeGeneration();
    $self->test_PaymentTypeGenerationWithAlreadyExistingAlias();
    done_testing();
}



sub test_PaymentTypeGenerationWithUndefParams {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethodGenerator = DE_EPAGES::Ipayment::API::PaymentMethodGenerator->new();
    my $PaymentMethod = undef;
    my $aSelectedPaymentMethods = undef;

    my $Status = $PaymentMethodGenerator->generatePaymentMethods( $PaymentMethod, $aSelectedPaymentMethods );
    is($Status, 0,  "$SubTestName - Tests if status flag is 0 if inputs are undef");

    my $aGeneratedPaymentMethods = $PaymentMethodGenerator->getGeneratedPaymentMethods();
    is_deeply($aGeneratedPaymentMethods, [], "$SubTestName - Tests if return array us empty if inputs are undef");
}



sub test_PaymentTypeGenerationWithEmptyPaymentMethodList {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethodGenerator = DE_EPAGES::Ipayment::API::PaymentMethodGenerator->new();
    my $PaymentMethod = $self->_getPaymentMethodIpayment();
    my $aSelectedPaymentMethods = [];

    my $Status = $PaymentMethodGenerator->generatePaymentMethods( $PaymentMethod, $aSelectedPaymentMethods );
    is($Status, 0,  "$SubTestName - Tests if status flag is 0 if input array is empty");

    my $aGeneratedPaymentMethods = $PaymentMethodGenerator->getGeneratedPaymentMethods();
    is_deeply($aGeneratedPaymentMethods, [], "$SubTestName - Tests if return array us empty if input array is empty");
}



sub test_PaymentTypeGeneration {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethodGenerator = DE_EPAGES::Ipayment::API::PaymentMethodGenerator->new();
    $PaymentMethodGenerator = Test::MockObject::Extends->new($PaymentMethodGenerator);

    $PaymentMethodGenerator->mock('_LoadObjectByPath', sub {
        return $self->_getPaymentTypeIpayment();
    });
    $PaymentMethodGenerator->mock('_LoadObjectByGUID', sub {
        return $self->_getPaymentTypeMethod();
    });
    $PaymentMethodGenerator->mock('_ExistsObjectByPath', sub {
        return 0;
    });
    $PaymentMethodGenerator->mock('_Transaction', sub {
        my $self = shift;
        my ($FunctionPointer) = @_;
        $FunctionPointer->();
        return;
    });

    my $PaymentMethod = $self->_getPaymentMethodIpayment();
    my $aSelectedPaymentMethods = ['314'];

    my $Status = $PaymentMethodGenerator->generatePaymentMethods( $PaymentMethod, $aSelectedPaymentMethods );
    is($Status, 1, "$SubTestName - Tests if status flag is 1 if valid inputs are given");

    my $aGeneratedPaymentMethods = $PaymentMethodGenerator->getGeneratedPaymentMethods();
    is(scalar @{ $aGeneratedPaymentMethods }, scalar @{ $aSelectedPaymentMethods }, "$SubTestName - Is number of generated payment method equal to number of selected");

    my $GeneratedPaymentMethod = $aGeneratedPaymentMethods->[0];
    is($GeneratedPaymentMethod->{'PaymentTypeMethodName'}, 'MyPaymentTypeMethod', "$SubTestName - test if payment type method of new type is correct");
    is($GeneratedPaymentMethod->{'NewPaymentMethod'}->get('Name'), 'Test Payment Type (MyPaymentTypeMethod)', "$SubTestName - test if new payment has correct name");
    is($GeneratedPaymentMethod->{'NewPaymentMethod'}->alias(), 'Test Payment Type (MyPaymentTypeMethod)', "$SubTestName - test if new payment has correct name");

}



sub test_PaymentTypeGenerationWithAlreadyExistingAlias {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethodGenerator = DE_EPAGES::Ipayment::API::PaymentMethodGenerator->new();
    $PaymentMethodGenerator = Test::MockObject::Extends->new($PaymentMethodGenerator);

    $PaymentMethodGenerator->mock('_LoadObjectByPath', sub {
        return $self->_getPaymentTypeIpayment();
    });
    $PaymentMethodGenerator->mock('_LoadObjectByGUID', sub {
        return $self->_getPaymentTypeMethod();
    });
    $PaymentMethodGenerator->mock('_ExistsObjectByPath', sub {
        return 1;
    });
    $PaymentMethodGenerator->mock('_Transaction', sub {
        my $self = shift;
        my ($FunctionPointer) = @_;
        $FunctionPointer->();
        return;
    });

    my $PaymentMethod = $self->_getPaymentMethodIpayment();
    my $aSelectedPaymentMethods = ['314'];

    my $Status = $PaymentMethodGenerator->generatePaymentMethods( $PaymentMethod, $aSelectedPaymentMethods );
    is($Status, 1, "$SubTestName - Tests if status flag is 1 if valid inputs are given");

    my $aGeneratedPaymentMethods = $PaymentMethodGenerator->getGeneratedPaymentMethods();
    is(scalar @{ $aGeneratedPaymentMethods }, scalar @{ $aSelectedPaymentMethods }, "$SubTestName - Is number of generated payment method equal to number of selected");

    my $GeneratedPaymentMethod = $aGeneratedPaymentMethods->[0];
    is($GeneratedPaymentMethod->{'PaymentTypeMethodName'}, 'MyPaymentTypeMethod', "$SubTestName - test if payment type method of new type is correct");
    is($GeneratedPaymentMethod->{'NewPaymentMethod'}->get('Name'), 'Test Payment Type (MyPaymentTypeMethod)', "$SubTestName - test if new payment has correct name");

    my $NewPaymentMethodAlias = $GeneratedPaymentMethod->{'NewPaymentMethod'}->alias();
    my $NewPaymentMethodAliasLength = length($NewPaymentMethodAlias);
    my $NewPaymentMethodNameWithGuid = $GeneratedPaymentMethod->{'NewPaymentMethod'}->get('Name').' '.GenerateShortGUID();
    my $NewPaymentMethodNameWithGuidLength = length($NewPaymentMethodNameWithGuid);

    is($NewPaymentMethodAliasLength, $NewPaymentMethodNameWithGuidLength, "$SubTestName - test if the alias is built from the name plus a GUID");

}



#----------------------------------------------------------------------------------------



sub _getPaymentTypeIpayment {
    my $self = shift;
    my $hAttributes = {
        'Name' => 'Test Payment Type'
    };
    my $MockedPaymentTypeIpayment = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Payment::API::Object::PaymentType', $hAttributes
    );
    $MockedPaymentTypeIpayment->mock('newPaymentMethod', sub {
        shift;
        my ($Shop, $hParameters) = @_;
        return $self->_getPaymentMethodIpayment($hParameters);
    });
    return $MockedPaymentTypeIpayment;
}



sub _getPaymentMethodIpayment {
    my $self = shift;
    my ($hParameters) = @_;

    my $hAttributes = {
        'ID' => '123',
        'PaymentType' => $self->_getPaymentTypeIpayment(),
        'AccountID' => '123',
        'UserID' => '123',
        'UserPassword' => '123',
        'AdminActionPassword' => '123',
        'SelectedTransactionType' => '123',
        'IsInTestMode' => '1',
        'IsIn3dSecureMode' => '1',
        'PaymentTypeMethod' => $self->_getPaymentTypeMethod(),
    };

    $hAttributes = merge($hAttributes, $hParameters);

    my $MockedPaymentMethodIpayment = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Ipayment::API::Object::PaymentMethodIpayment', $hAttributes
    );
    $MockedPaymentMethodIpayment->mock('getSite', sub {
        return $self->_getSite();
    });
    return $MockedPaymentMethodIpayment;
}



sub _getSite {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'MyShop',
        'ShippingMethods' => [],
    };
    my $MockedSite = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );
    return $MockedSite;
}



sub _getPaymentTypeMethod {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'MyPaymentTypeMethod',
        'NameOrAlias' => 'MyPaymentTypeMethod',
        'Options3dSecure' => '{"Options3dSecure":["1","0"]}',
        'TransactionTypes' => '{"TransactionTypes":["auth","preauth"]}',
    };
    my $MockedPaymentTypeMethod = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::ExternalPayment::API::PaymentTypeMethods', $hAttributes
    );
    $MockedPaymentTypeMethod->mock('newPaymentMethod', sub {
        return $self->_getPaymentMethodIpayment();
    });
    return $MockedPaymentTypeMethod;
}

__PACKAGE__->runScript();