#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::TestTools
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for TestTools
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::TestTools;
use base DE_EPAGES::Core::API::Test;
use strict;



use Test::MockObject;
use Test::More;
use Test::Exception;
use DE_EPAGES::Ipayment::t::unit::TestTools qw (
    TestExistsMethod
    CreatePositiveXMLHttpResponse
    CreateNegativeXMLHttpResponse
    CreateRedirectHttpResponse
    CreateMockedUserAgentForPostRequests
    CreateMockedUserAgent
    IsXml
    NormalizeXmlString
);



sub test {
    my $self = shift;
    $self->test_TestExistsMethod();
    $self->test_TestExistsMethodNegative();

    $self->test_CreatePositiveXMLHttpResponse();
    $self->test_CreateNegativeXMLHttpResponse();
    $self->test_CreateRedirectHttpResponse();

    $self->test_CreateMockedUserAgentForPostRequests();
    $self->test_CreateMockedUserAgent();

    $self->test_IsXml();
    $self->test_IsXmlNegative();
    $self->test_NormalizeXmlString();
    done_testing();
}



sub test_CreatePositiveXMLHttpResponse() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Content = 'testContent';
    my $PositiveResponse = CreatePositiveXMLHttpResponse($Content);

    my $ExpectedContent = 'testContent';
    my $ExpectedMsg = 'OK';
    my $ExpectedRC = 200; ## no critic (ProhibitMagicNumbers)

    is($PositiveResponse->{'_content'},$ExpectedContent,"$SubTestName - Test if the content was set.");
    is($PositiveResponse->{'_msg'},$ExpectedMsg,"$SubTestName - Test if the msg was $ExpectedMsg.");
    is($PositiveResponse->{'_rc'},$ExpectedRC,"$SubTestName - Test if the response rc was $ExpectedRC.");
}



sub test_CreateNegativeXMLHttpResponse() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Content = 'testContent';
    my $NegativeResponse = CreateNegativeXMLHttpResponse($Content);

    my $ExpectedContent = '';
    my $ExpectedMsg = 'Internal Server Error';
    my $ExpectedRC = 500; ## no critic (ProhibitMagicNumbers)

    is($NegativeResponse->{'_content'},$ExpectedContent,"$SubTestName - Test if the content is empty.");
    is($NegativeResponse->{'_msg'},$ExpectedMsg,"$SubTestName - Test if the msg was $ExpectedMsg.");
    is($NegativeResponse->{'_rc'},$ExpectedRC,"$SubTestName - Test if the response rc was $ExpectedRC.");
}



sub test_CreateRedirectHttpResponse() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Content = 'testContent';
    my $RedirectResponse = CreateRedirectHttpResponse($Content);

    my $ExpectedContent = '';
    my $ExpectedMsg = 'Found';
    my $ExpectedRC = 302; ## no critic (ProhibitMagicNumbers)

    is($RedirectResponse->{'_content'},$ExpectedContent,"$SubTestName - Test if the content is empty.");
    is($RedirectResponse->{'_msg'},$ExpectedMsg,"$SubTestName - Test if the msg was $ExpectedMsg.");
    is($RedirectResponse->{'_rc'},$ExpectedRC,"$SubTestName - Test if the response rc was $ExpectedRC.");
}



sub test_CreateMockedUserAgentForPostRequests() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Response = HTTP::Response->new( 123 ); ## no critic (ProhibitMagicNumbers)
    my $UserAgent = CreateMockedUserAgentForPostRequests($Response);

    is_deeply($UserAgent->post(), $Response, "$SubTestName - Test if the response is return from the userAgent.");
}



sub test_CreateMockedUserAgent() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Response = HTTP::Response->new( 123 ); ## no critic (ProhibitMagicNumbers)
    my $UserAgent = CreateMockedUserAgent($Response);

    is_deeply($UserAgent->request(), $Response, "$SubTestName - Test if the response is return from the userAgent.");
}



sub test_IsXml(){
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $validXML = '<?xml version="1.0" encoding="UTF-8"?><validTag>somethingToTest</validTag>';
    my $IsXml = IsXml($validXML);

    my $ExpectedIsXml = 1;
    is($IsXml,$ExpectedIsXml,"$SubTestName - Test if the IsXml is returning valid for a XML.");
}



sub test_IsXmlNegative(){
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $InvalidXML = '<?xml version="1.0" encoding="UTF-8"?><invalidTag>somethingToTest<</invalidTag>';
    my $IsXml = IsXml($InvalidXML);

    my $ExpectedIsXml = 0;
    is($IsXml,$ExpectedIsXml,"$SubTestName - Test if the IsXml is returning 0 for an invalid XML.");
}



sub test_TestExistsMethod {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $TestClass = Test::MockObject->new();
    $TestClass->mock('TestMethod', {});

    lives_ok(
        sub { TestExistsMethod($TestClass,'TestMethod') },
        "$SubTestName - tests if the TestExistsMethod works fine if the Method exists"
    );
}



sub test_NormalizeXmlString {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

## no critic (ProhibitImplicitNewlines) it tests if it can cleanup the string
    my $XML = '<?xml version="1.0"           encoding="UTF-8"    
           ?>     
    <validTag            > ObjectID=123    </validTag>';
## use critic (ProhibitImplicitNewlines)
    my $NormalizeXML = NormalizeXmlString($XML);
    my $ExpectedXML = '<?xml version="1.0" encoding="UTF-8" ?><validTag > ObjectID=replaced</validTag>';

    is($NormalizeXML,$ExpectedXML,"$SubTestName - Test if the Xml has been Normalized.");
#    print $NormalizeXML;
}



sub test_TestExistsMethodNegative {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $TestClass = Test::MockObject->new();
    $TestClass->mock('TestMethod', {});

    throws_ok { TestExistsMethod($TestClass,'OtherMethod') } (
        qr/Test::MockObject don't have a method like: OtherMethod at/,
        "$SubTestName - tests if Calling of a non existing method is found by TestExistsMethod"
    );
}



__PACKAGE__->runScript();
1;