#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI/Shop.pm
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Ipayment::API::CGI::Response;
use DE_EPAGES::Ipayment::API::CGI::SecurityLayer;
use DE_EPAGES::Ipayment::API::Constants qw ( $ExpectedReturnIps );
use DE_EPAGES::Ipayment::t::unit::TestTools qw ( TestExistsMethod );
use DE_EPAGES::Ipayment::UI::Shop;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::MockObject;
use Test::MockObject::Extends;
use Test::More;

use strict;

use constant CorrectIpAddress => $$ExpectedReturnIps[0]; # get the first number from the expected IPs

#----------------------------------------------------------------------------------------
sub test {
    my $self = shift;
    $self->test_PaymentHiddenTriggerIpaymentWithInvalidData();
    $self->test_PaymentHiddenTriggerIpaymentWithValidDataSuccessfulTransaction();
    $self->test_PaymentHiddenTriggerIpaymentWithValidDataNotSuccessfulTransaction();
    $self->test_PaymentHiddenTriggerIpaymentSecurityLayerThrowingError();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_PaymentHiddenTriggerIpaymentSecurityLayerThrowingError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SecurityLayerObject = $self->_createCGISecurityThrowingError();
    my $Shop = $self->_createShop($SecurityLayerObject);
    my $Servlet = $self->_createServlet();

    $Shop->PaymentHiddenTriggerIpayment($Servlet);

    my $ExpectedErrorCode = 'SecurityLayerFail';
    my $Error = $Shop->{'LogPaymentParams_MockedVar'}[2];
    is($Error->code(), $ExpectedErrorCode,
      "$SubTestName - test if data sent to LogPayment it is correct when a non ipayment IP address is calling HiddenTrigger" );

    return;
}
#----------------------------------------------------------------------------------------
sub test_PaymentHiddenTriggerIpaymentWithInvalidData {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsValidRequest = 0;
    my $ExpectedErrorMessage = 'Error: ErrorMessage';
    my $SecurityLayerObject = $self->_createCGISecurityLayerMock($IsValidRequest, $ExpectedErrorMessage);
    my $WasTransactionSuccessfulCreated = 0;
    my $Shop = $self->_createShop($SecurityLayerObject,$WasTransactionSuccessfulCreated);
    my $WrongIpAddress = '123.456.789.123';
    my $Servlet = $self->_createServlet($WrongIpAddress);
    my $ReturnValue = $Shop->PaymentHiddenTriggerIpayment($Servlet);

    is($ReturnValue, undef, "$SubTestName - test if we return undef from calling HiddenTrigger");

    my $Servlet_BlockMonitor = ${$Shop->{'BlockMonitor_MockedVar'}}[0]; #Get the first element of the parament list of BlockMonitor
    ok($Servlet_BlockMonitor->can('params'), "$SubTestName - test if the first element of the parameter list is a Servlet which support params()" );

    my $ExpectedLogPaymentParams = [
          'Ipayment',
          'Request Error: invalid params',
          $ExpectedErrorMessage
    ];
    is_deeply($Shop->{'LogPaymentParams_MockedVar'},
                $ExpectedLogPaymentParams,
                "$SubTestName - test if data sent to LogPayment it is correct when a non ipayment IP address is calling HiddenTrigger"
    );
    my $ExpectedFormDataParams = [ {
            'shopper_id' => '53DB8B33-D3CF-CDD0-9020-D5809AB2ABF8',
            'REMOTE_ADDR' => $WrongIpAddress
          } ];
    is_deeply($SecurityLayerObject->{'extractDataFromParams_MockVar'},
                $ExpectedFormDataParams,
                "$SubTestName - compare the method arguments which were sent to extractDataFromParams"
    );
    my $ExpectedErrorMessageParams = [];
    is_deeply($SecurityLayerObject->{'getErrorMessageParams_MockVar'},
                $ExpectedErrorMessageParams,
                "$SubTestName - compare the method arguments which were sent to extractDataFromParams"
    );
}

#----------------------------------------------------------------------------------------
sub test_PaymentHiddenTriggerIpaymentWithValidDataSuccessfulTransaction {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsValidRequest = 1;
    my $ExpectedErrorMessage = '';
    my $SecurityLayerObject = $self->_createCGISecurityLayerMock($IsValidRequest, $ExpectedErrorMessage);
    my $WasTransactionSuccessfulCreated = 1;
    my $Shop = $self->_createShop($SecurityLayerObject,$WasTransactionSuccessfulCreated);
    my $Servlet = $self->_createServlet(CorrectIpAddress);

    my $ReturnValue = $Shop->PaymentHiddenTriggerIpayment($Servlet);

    is($ReturnValue, undef, "$SubTestName - test if we return undef from calling HiddenTrigger");

    my $Servlet_BlockMonitor = ${$Shop->{'BlockMonitor_MockedVar'}}[0]; #Get the first element of the parament list of BlockMonitor
    ok($Servlet_BlockMonitor->can('params'), "$SubTestName - test if the first element of the parameter list is a Servlet which support params()" );

    my $ExpectedFormDataParams = [ {
            'shopper_id' => '53DB8B33-D3CF-CDD0-9020-D5809AB2ABF8',
            'REMOTE_ADDR' => CorrectIpAddress
    } ];
    is_deeply($SecurityLayerObject->{'extractDataFromParams_MockVar'},
                $ExpectedFormDataParams,
                "$SubTestName - compare the method arguments which were sent to extractDataFromParams"
    );
    my $ExpectedLogPaymentParams = [
        'Ipayment',
        'Correct HiddenTrigger call, from Ipayment IP. Parameters',
        {
          'shopper_id' => '53DB8B33-D3CF-CDD0-9020-D5809AB2ABF8',
          'REMOTE_ADDR' => CorrectIpAddress
        } ];
    is_deeply($Shop->{'LogPaymentParams_MockedVar'},
                $ExpectedLogPaymentParams,
                "$SubTestName - test if data sent to LogPayment it is correct when an ipayment IP address is calling HiddenTrigger"
    );
    my @DoBasketOrOrderParams = @{$Shop->{'_DoBasket2Order_MockedVar'}}; #Get input params from the _DoBasket2Order method
    my $Servlet_DoBasketOrOrder = $DoBasketOrOrderParams[0];
    my $Basket = $DoBasketOrOrderParams[1];
    ok($Servlet_DoBasketOrOrder->can('params'), "$SubTestName - check the first param from _DoBasket2Order method - servlet object" );
    is_deeply($Servlet_DoBasketOrOrder->{'writeContentOutputParams_MockVar'},
        ['OK'],
        "$SubTestName - check if the writeContentOutput(OK) was write on the servlet"
    );
    is ($Basket->get('instanceOf'), 'Basket',"$SubTestName - check the second param from _DoBasket2Order method - basket object");
    my $SecurityLayer_createTransaction = ${$self->{'createTransaction_MockVar'}}[0];
    isa_ok($SecurityLayer_createTransaction,
        'DE_EPAGES::Ipayment::API::CGI::SecurityLayer',
        "$SubTestName - check if the first input value of createTransaction is a SecurityLayer object"
    );
}

#----------------------------------------------------------------------------------------
sub test_PaymentHiddenTriggerIpaymentWithValidDataNotSuccessfulTransaction {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsValidRequest = 1;
    my $ExpectedErrorMessage = '';
    my $SecurityLayerObject = $self->_createCGISecurityLayerMock($IsValidRequest, $ExpectedErrorMessage);
    my $WasTransactionNOTSuccessfulCreated = 0;
    my $Shop = $self->_createShop($SecurityLayerObject,$WasTransactionNOTSuccessfulCreated);
    my $Servlet = $self->_createServlet(CorrectIpAddress);

    my $ReturnValue = $Shop->PaymentHiddenTriggerIpayment($Servlet);

    is($ReturnValue, undef, "$SubTestName - test if we return undef from calling HiddenTrigger");

    my $Servlet_BlockMonitor = ${$Shop->{'BlockMonitor_MockedVar'}}[0]; #Get the first element of the parament list of BlockMonitor
    ok($Servlet_BlockMonitor->can('params'), "$SubTestName - test if the first element of the parameter list is a Servlet which support params()" );

    my $ExpectedFormDataParams = [ {
            'shopper_id' => '53DB8B33-D3CF-CDD0-9020-D5809AB2ABF8',
            'REMOTE_ADDR' => CorrectIpAddress
    } ];
    is_deeply($SecurityLayerObject->{'extractDataFromParams_MockVar'},
                $ExpectedFormDataParams,
                "$SubTestName - compare the method arguments which were sent to extractDataFromParams"
    );
    my $ExpectedLogPaymentParams = [
        'Ipayment',
        'Correct HiddenTrigger call, from Ipayment IP. Parameters',
        {
          'shopper_id' => '53DB8B33-D3CF-CDD0-9020-D5809AB2ABF8',
          'REMOTE_ADDR' => CorrectIpAddress
        } ];
    is_deeply($Shop->{'LogPaymentParams_MockedVar'},
                $ExpectedLogPaymentParams,
                "$SubTestName - test if data sent to LogPayment it is correct when an ipayment IP address is calling HiddenTrigger"
    );

    my $ExpectedParamsOf_getBasketOrOrder = undef;
    my $ParamsOf_getBasketOrOrder = $Shop->{'getBasketOrOrder_MockVar'};
    is ($ParamsOf_getBasketOrOrder, $ExpectedParamsOf_getBasketOrOrder,"$SubTestName - Test if the method getBasketOrOrder was not called");
}

#----------------------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;
    my ($IPAddress) = @_;

    my $ServletMock = Test::MockObject->new();
    $ServletMock->mock('form', sub{});

    $ServletMock->mock('params', sub{
        my $hParams = {
            'REMOTE_ADDR' => $IPAddress,
            'shopper_id' => '53DB8B33-D3CF-CDD0-9020-D5809AB2ABF8',
        };
        return $hParams;
    });
    $ServletMock->mock('writeContentOutput', sub{
        shift;
        $ServletMock->{'writeContentOutputParams_MockVar'} = \@_;
    });
    return $ServletMock;
}
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my ($SecurityLayer, $WasTransactionSuccessfulCreated) = @_;
    my $Shop = DE_EPAGES::Ipayment::UI::Shop->new();
    my $MockedShop = Test::MockObject::Extends->new($Shop);

    TestExistsMethod($Shop, '_LogPayment');
    $MockedShop->mock('_LogPayment', sub{
        shift;
        $MockedShop->{'LogPaymentParams_MockedVar'} = \@_;
    });

    TestExistsMethod($Shop, 'BlockMonitor');
    $MockedShop->mock('BlockMonitor', sub{
        shift;
        $MockedShop->{'BlockMonitor_MockedVar'} = \@_;
    });
    TestExistsMethod($Shop, '_DoBasket2Order');
    TestExistsMethod('DE_EPAGES::Order::UI::Basket', 'doBasket2Order');
    $MockedShop->mock('_DoBasket2Order', sub{
        shift;
        $MockedShop->{'_DoBasket2Order_MockedVar'} = \@_;
    });

    TestExistsMethod($Shop, '_getCGISecurityLayer');
    $MockedShop->mock('_getCGISecurityLayer', sub{
        return $SecurityLayer;
    });

    TestExistsMethod($Shop, '_createCGIResponse');
    $MockedShop->mock('_createCGIResponse', sub{
        return $self->_createCGIResponse($WasTransactionSuccessfulCreated);
    });

    return $MockedShop;
}
sub _createCGIResponse {
    my $self = shift;
    my ($WasTransactionSuccessfulCreated) = @_;
    my $Response = DE_EPAGES::Ipayment::API::CGI::Response->new();
    my $MockedResponse = Test::MockObject::Extends->new($Response);
    TestExistsMethod($MockedResponse, 'createTransaction');
    $MockedResponse->mock('createTransaction', sub{
        shift;
        $self->{'createTransaction_MockVar'} = \@_;
        return $WasTransactionSuccessfulCreated;
    });

    return $MockedResponse;
}
#----------------------------------------------------------------------------------------
sub _createCGISecurityThrowingError {
    my $self = shift;
    my $SecurityLayer = DE_EPAGES::Ipayment::API::CGI::SecurityLayer->new();
    my $MockedSecurityLayer = Test::MockObject::Extends->new($SecurityLayer);

    TestExistsMethod($SecurityLayer, 'extractDataFromParams');
    $MockedSecurityLayer->mock('extractDataFromParams', sub{
        Error('SecurityLayerFail');
    });
    return $MockedSecurityLayer;
}

#----------------------------------------------------------------------------------------

sub _createCGISecurityLayerMock {
    my $self = shift;
    my ($ReturnValue_extractDataFromParams, $ReturnValue_getErrorMessage) = @_;
    my $SecurityLayer = DE_EPAGES::Ipayment::API::CGI::SecurityLayer->new();
    my $MockedSecurityLayer = Test::MockObject::Extends->new($SecurityLayer);

    TestExistsMethod($SecurityLayer, 'extractDataFromParams');
    $MockedSecurityLayer->mock('extractDataFromParams', sub{
        shift;
        $MockedSecurityLayer->{'extractDataFromParams_MockVar'} = \@_;
        return $ReturnValue_extractDataFromParams;
    });
    TestExistsMethod($SecurityLayer, 'getErrorMessage');
    $MockedSecurityLayer->mock('getErrorMessage', sub{
        shift;
        $MockedSecurityLayer->{'getErrorMessageParams_MockVar'} = \@_;
        return $ReturnValue_getErrorMessage;
    });
    TestExistsMethod($SecurityLayer, 'getBasketOrOrder');
    $MockedSecurityLayer->mock('getBasketOrOrder', sub{
        shift;
        $MockedSecurityLayer->{'getBasketOrOrder_MockVar'} = \@_;
        return $self->_createBasketObject();
    });
    return $MockedSecurityLayer;
}

sub _createBasketObject {
    my $self = shift;
    my $hAtributes = {'instanceOf' => 'Basket'};
    my $BasketOrOrder = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket',$hAtributes);
    return $BasketOrOrder;
}
__PACKAGE__->runScript();
1;